<?php
/**
 * Poynt — a GoDaddy Brand for WooCommerce
 *
 * @author GoDaddy
 * @copyright Copyright (c) 2021 GoDaddy Operating Company, LLC. All Rights Reserved.
 * @license GPL-2.0
 */

namespace GoDaddy\WooCommerce\Poynt\Gateways;

use GoDaddy\WooCommerce\Poynt\API\Cards\TokenizeChargeResponse;
use GoDaddy\WooCommerce\Poynt\API\GatewayAPI;
use GoDaddy\WooCommerce\Poynt\API\Cards\TokenizeResponse;
use GoDaddy\WooCommerce\Poynt\Helpers\CredentialsHelper;
use GoDaddy\WooCommerce\Poynt\Plugin;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3 as Framework;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3\SV_WC_Payment_Gateway_API_Customer_Response;
use stdClass;
use WC_HTTPS;
use WC_Order;

defined('ABSPATH') or exit;

/**
 * Credit card gateway handler.
 *
 * @since 1.0.0
 *
 * @method Plugin get_plugin()
 * @method PaymentForm|null get_payment_form_instance()
 */
class CreditCardGateway extends Framework\SV_WC_Payment_Gateway_Direct
{
    /** @var string additional card type used by this gateway */
    const CARD_TYPE_UNIONPAY = 'unionpay';

    /** @var GatewayAPI instance */
    private $api;

    /** @var string the configured production app ID */
    protected $appId;

    /** @var string the configured production private key */
    protected $privateKey;

    /** @var string the configured staging app ID */
    protected $stagingAppId;

    /** @var string the configured staging private key */
    protected $stagingPrivateKey;

    /**
     * Credit card gateway constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        $cardTypeOptions = [];
        $supportedCardTypes = [
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_AMEX,
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_DINERSCLUB,
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_DISCOVER,
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_JCB,
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_MASTERCARD,
            self::CARD_TYPE_UNIONPAY,
            Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_VISA
        ];

        foreach ($supportedCardTypes as $supportedCardType) {
            $cardTypeOptions[$supportedCardType] = Framework\SV_WC_Payment_Gateway_Helper::payment_type_to_name($supportedCardType);
        }

        $plugin = poynt_for_woocommerce();

        parent::__construct(
            Plugin::CREDIT_CARD_GATEWAY_ID,
            $plugin,
            [
                'method_title'       => esc_html__('Credit Card', 'godaddy-payments'),
                'method_description' => esc_html__('Securely accept credit card payments on your checkout page with Poynt &mdash; a GoDaddy brand.', 'godaddy-payments'),
                'payment_type'       => self::PAYMENT_TYPE_CREDIT_CARD,
                'card_types'         => $cardTypeOptions,
                'environments'       => [
                    $plugin::ENVIRONMENT_PRODUCTION => esc_html_x('Production', 'Gateway environment', 'godaddy-payments'),
                    $plugin::ENVIRONMENT_STAGING    => esc_html_x('Staging', 'Gateway environment', 'godaddy-payments'),
                ],
                'supports' => [
                    // @TODO review these feature flags before release as we move forward with other stories {FN 2020-02-04}
                    self::FEATURE_PRODUCTS,
                    self::FEATURE_CARD_TYPES,
                    self::FEATURE_PAYMENT_FORM,
                    self::FEATURE_CREDIT_CARD_CHARGE,
                    self::FEATURE_CREDIT_CARD_CHARGE_VIRTUAL,
                    self::FEATURE_CREDIT_CARD_AUTHORIZATION,
                    self::FEATURE_CREDIT_CARD_CAPTURE,
                    self::FEATURE_DETAILED_CUSTOMER_DECLINE_MESSAGES,
                    self::FEATURE_REFUNDS,
                    self::FEATURE_VOIDS,
                    self::FEATURE_TOKENIZATION,
                    self::FEATURE_ADD_PAYMENT_METHOD,
                    self::FEATURE_TOKEN_EDITOR,
                ],
            ]
        );

        add_filter("woocommerce_settings_api_sanitized_fields_{$this->get_id()}", [$this, 'formatSanitizedFields']);
    }

    /**
     * Gets the gateway API instance.
     *
     * @since 1.0.0
     *
     * @return GatewayAPI
     */
    public function get_api() : GatewayAPI
    {
        if (! $this->api instanceof GatewayAPI) {

            $environment = $this->get_environment();

            $this->api = new GatewayAPI($this->getAppId(), $this->getBusinessId(), $this->getPrivateKey(), is_string($environment) ? $environment : '');
        }

        return $this->api;
    }

    /**
     * Gets a credential based on the environment.
     *
     * @since 1.0.0
     *
     * @param string $credentialKey key of the credential to get
     * @param string $environment the desired environment, defaults to the configured setting
     * @return string
     */
    protected function getCredential(string $credentialKey, string $environment = '') : string
    {
        if (! $environment) {
            $environment = $this->get_environment();
        }

        // prefix with the environment if not production
        if (Plugin::ENVIRONMENT_PRODUCTION !== $environment) {
            $credentialKey = $environment.ucfirst($credentialKey);
        }

        $credential = $this->$credentialKey ?? '';

        return is_string($credential) ? $credential : '';
    }

    /**
     * Gets the configured app ID for the given environment.
     *
     * @since 1.0.0
     *
     * @param string $environment optional environment, defaults to current configured environment
     * @return string
     */
    public function getAppId(string $environment = '') : string
    {
        $credential = $this->getCredential('appId', $environment);
        $credentialPieces = explode('=', $credential);

        return end($credentialPieces);
    }

    /**
     * Gets the configured business ID for the given environment.
     *
     * @since 1.0.0
     *
     * @param string $environment optional environment, defaults to current configured environment
     * @return string
     */
    public function getBusinessId(string $environment = '') : string
    {
        $credential = $this->getCredential('appId', $environment);
        $credentialPieces = explode('=', $credential);

        return current($credentialPieces);
    }

    /**
     * Gets the configured private key for the given environment.
     *
     * @since 1.0.0
     *
     * @param string $environment optional environment, defaults to current configured environment
     * @return string
     */
    public function getPrivateKey(string $environment = '') : string
    {
        return $this->getCredential('privateKey', $environment);
    }

    /**
     * Gets the nonce token from posted data.
     *
     * @since 1.0.0
     *
     * @return string
     */
    private function getNonce() : string
    {
        $nonce = Framework\SV_WC_Helper::get_posted_value('wc_'.$this->get_id().'_nonce', null);

        return is_string($nonce) ? $nonce : '';
    }

    /**
     * Determines if a nonce was posted.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    private function hasNonce() : bool
    {
        return (bool) $this->getNonce();
    }

    /**
     * Gets an order object with additional payment data.
     *
     * @since 1.0.0
     *
     * @param int|WC_Order $order
     * @return WC_Order
     */
    public function get_order($order) : WC_Order
    {
        $order = parent::get_order($order);

        if (! isset($order->payment)) {
            $order->payment = new stdClass();
        }

        if ($nonce = $this->getNonce()) {
            $order->payment->nonce = wc_clean($nonce);
        }

        return $order;
    }

    /**
     * Gets the order ready for a refund.
     *
     * @since 1.0.0
     *
     * @param int|WC_Order $order  order object or ID
     * @param float        $amount amount to refund
     * @param string       $reason refund reason
     *
     * @return WC_Order|\WP_Error
     */
    protected function get_order_for_refund($order, $amount, $reason)
    {
        $order = parent::get_order_for_refund($order, $amount, $reason);

        // if there was a capture ID, use that for the refund
        if ($order instanceof WC_Order && ($captureTransactionId = $this->get_order_meta($order, 'capture_trans_id'))) {
            $order->refund->trans_id = $captureTransactionId;
        }

        return $order;
    }

    /**
     * Gets the payment gateway customer ID.
     *
     * @since 1.0.0
     *
     * @param int $userId user ID
     * @param array $args array of arguments
     * @return string|bool
     */
    public function get_customer_id($userId, $args = [])
    {
        // not available for this gateway - implements parent method
        return false;
    }

    /**
     * Gets the payment gateway guest customer ID.
     *
     * @since 1.0.0
     *
     * @param WC_Order $order order object
     * @return string|bool
     */
    public function get_guest_customer_id(WC_Order $order)
    {
        // not available for this gateway - implements parent method
        return false;
    }

    /**
     * Formats sanitized fields for credentials.
     *
     * @internal
     *
     * @since 1.0.0
     *
     * @param array $sanitized_fields array of sanitized fields
     * @return array
     */
    public function formatSanitizedFields(array $sanitized_fields) : array
    {
        if (isset( $sanitized_fields['privateKey'])) {
            $sanitized_fields['privateKey'] = CredentialsHelper::formatPrivateKey($sanitized_fields['privateKey']);
        }

        if (isset($sanitized_fields['stagingPrivateKey'])) {
            $sanitized_fields['stagingPrivateKey'] = CredentialsHelper::formatPrivateKey($sanitized_fields['stagingPrivateKey']);
        }

        return $sanitized_fields;
    }

    /**
     * Gets a link to Poynt account admin to obtain credentials for a given environment.
     *
     * @since 1.0.0
     *
     * @param string $environment optional environment variable, defaults to configured
     * @return string will default to production if an invalid environment is given
     */
    private function getCredentialsUrl(string $environment = '') : string
    {
        if ('' === $environment) {
            $environment = $this->get_environment();
        }

        switch($environment) {
            case Plugin::ENVIRONMENT_STAGING :
                return 'https://st.poynt.net/settings/business/contact';
            case Plugin::ENVIRONMENT_PRODUCTION :
            default:
                return 'https://poynt.net/settings/business/contact';
        }
    }

    /**
     * Gets the credit card form fields.
     *
     * @since 1.0.0
     *
     * @return array
     */
    protected function get_method_form_fields() : array
    {
        return [
            'connection_settings' => [
                'type'        => 'title',
                'title'       => __('Connection settings', 'godaddy-payments'),
                'description' => sprintf(
                    /** translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                    __('To connect to Poynt, please copy your Application ID and Private Key and paste these values in the fields below. %1$sClick here%2$s to access these credentials.', 'godaddy-payments'),
                    '<a id="woocommerce_'.esc_attr($this->get_id()).'_credentials_link" href="'.esc_url($this->getCredentialsUrl()).'" target="_blank">', '</a>'
                ),
            ],
            'appId'               => [
                'title'    => __('Application ID', 'godaddy-payments'),
                'type'     => 'text',
                'desc_tip' => __('Enter your Application ID', 'godaddy-payments'),
                'class'    => 'environment-field production-field',
            ],
            'privateKey'          => [
                'title'    => __('Private Key', 'godaddy-payments'),
                'type'     => 'textarea',
                'desc_tip' => __('Enter your Private Key', 'godaddy-payments'),
                'class'    => 'environment-field production-field',
            ],
            'stagingAppId'        => [
                'title'    => __('Staging Application ID', 'godaddy-payments'),
                'type'     => 'text',
                'desc_tip' => __('Enter your staging Application ID', 'godaddy-payments'),
                'class'    => 'environment-field staging-field',
            ],
            'stagingPrivateKey'   => [
                'title'    => __('Staging Private Key', 'godaddy-payments'),
                'type'     => 'textarea',
                'desc_tip' => __('Enter your staging Private Key', 'godaddy-payments'),
                'class'    => 'environment-field staging-field',
            ]
        ];
    }

    /**
     * Adds credit card authorization and charge fields.
     *
     * @since 1.0.0
     *
     * @param array $formFields
     * @return array
     */
    protected function add_authorization_charge_form_fields($formFields) : array
    {
        $formFields = parent::add_authorization_charge_form_fields($formFields);

        $formFields['enable_paid_capture']['default'] = 'yes';

        return $formFields;
    }

    /**
     * Adds the enable Card Security Code form fields.
     *
     * @since 1.0.0
     *
     * @param array $formFields
     *
     * @return array
     */
    protected function add_csc_form_fields($formFields) : array
    {
        $formFields = parent::add_csc_form_fields($formFields);

        unset($formFields['card-csc'], $formFields['enable_csc']);

        if (isset($formFields['enable_token_csc'])) {
            unset($formFields['enable_token_csc']);
        }

        return $formFields;
    }

    /**
     * Adds gateway-specific transaction data to the order.
     *
     * @since 1.0.0
     *
     * @param \WC_Order $order the order object
     * @param SV_WC_Payment_Gateway_API_Customer_Response $response the transaction response
     */
    public function add_payment_gateway_transaction_data($order, $response) {
        if (TokenizeChargeResponse::CAPTURED_STATUS !== $response->get_status_code()) {

            $this->update_order_meta( $order, 'charge_captured', 'no' );
        }
    }

    /**
     * Displays the settings page and injects some inline JavaScript for handling some custom fields.
     *
     * @internal
     *
     * @since 1.0.0
     */
    public function admin_options()
    {
        parent::admin_options();

        $productionUrl = $this->getCredentialsUrl(Plugin::ENVIRONMENT_PRODUCTION);
        $stagingUrl = $this->getCredentialsUrl(Plugin::ENVIRONMENT_STAGING);

        wc_enqueue_js("
            $( '#woocommerce_".esc_js($this->get_id())."_environment' ).on( 'change', function( e ) {
                if ( 'staging' === $(this).find('option:selected').val() ) {
                    $( '#woocommerce_".esc_js($this->get_id())."_credentials_link' ).attr( 'href', '".esc_js($stagingUrl)."' );
                } else {
                    $( '#woocommerce_".esc_js($this->get_id())."_credentials_link' ).attr( 'href', '".esc_js($productionUrl)."' );
                }
            } ).trigger( 'change' );
        ");
    }

    /**
     * Flags whether the CSC feature is enabled.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    public function csc_enabled() : bool
    {
        return true;
    }

    /**
     * Flags whether the CSC feature should be enabled for tokens.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    public function csc_enabled_for_tokens() : bool
    {
        return false;
    }

    /**
     * Determines whether the gateway is configured.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    public function is_configured() : bool
    {
        return CredentialsHelper::isAppIdValid($this->getAppId()) && CredentialsHelper::isBusinessIdValid($this->getBusinessId());
    }

    /**
     * Flags whether tokenization takes place prior to transaction processing.
     *
     * @since 1.0.0
     *
     * @return bool
     */
    public function tokenize_before_sale() : bool
    {
        return true;
    }

    /**
     * Performs a credit card transaction for the given order and returns the result.
     *
     * @since 1.0.0
     *
     * @param WC_Order $order
     * @param null $response
     * @return Framework\SV_WC_Payment_Gateway_API_Response|TokenizeResponse
     * @throws Framework\SV_WC_Plugin_Exception
     */
    protected function do_credit_card_transaction($order, $response = null) : Framework\SV_WC_Payment_Gateway_API_Response
    {
        if (! $response && $order instanceof WC_Order && empty($order->payment->token)) {

            $tokenize_response = $this->get_api()->tokenize_payment_method($order);

            if (! $tokenize_response->transaction_approved()) {
                return $tokenize_response;
            }

            $token = $tokenize_response->get_payment_token();

            $order->payment->token     = $token->get_id();
            $order->payment->card_type = $token->get_card_type();
            $order->payment->exp_month = $token->get_exp_month();
            $order->payment->exp_year  = $token->get_exp_year();

            $order->payment->last_four = $order->payment->account_number = $token->get_last_four();

            return parent::do_credit_card_transaction($order);
        }

        return parent::do_credit_card_transaction($order, $response);
    }

    /**
     * Determines if the posted credit card fields are valid or not.
     *
     * @since 1.0.0
     *
     * @param bool $valid whether credit card fields are valid
     * @return bool
     */
    protected function validate_credit_card_fields($valid) : bool
    {
        return $this->hasNonce();
    }

    /**
     * Initializes the payment form instance.
     *
     * @since 1.0.0
     *
     * @return PaymentForm
     */
    protected function init_payment_form_instance() : PaymentForm
    {
        return new PaymentForm($this);
    }

    /**
     * Enqueues the payment form scripts and styles.
     *
     * @since 1.0.0
     */
    protected function enqueue_gateway_assets()
    {
        if ( is_account_page() && ! is_add_payment_method_page() ) {
            return;
        }

        switch($this->get_environment()) {
            case Plugin::ENVIRONMENT_PRODUCTION :
                $url = 'https://poynt.net/snippet/poynt-collect/bundle.js';
                break;
            case Plugin::ENVIRONMENT_STAGING :
                $url = 'https://st.poynt.net/snippet/poynt-collect/bundle-st.js';
                break;
        }

        if (empty($url)) {
            return;
        }

        wp_enqueue_script('poynt-collect', $url, [], Plugin::VERSION, true);

        parent::enqueue_gateway_assets();
    }

    /**
     * Gets the payment form localized script variables.
     *
     * @since 1.0.0
     *
     * @return array
     */
    protected function get_payment_form_js_localized_script_params() : array
    {
        $params = parent::get_payment_form_js_localized_script_params();

        if (! isset($params['generic_error'])) {
            $params['generic_error'] = __('An error occurred, please try again or try an alternate form of payment.', 'godaddy-payments');
        }

        if (! isset($params['missing_card_details'])) {
            $params['missing_card_details'] = __('Missing card details.', 'godaddy-payments');
        }

        if (! isset($params['missing_billing_fields'])) {
            $params['missing_billing_fields'] = __('Missing billing details.', 'godaddy-payments');
        }

        return $params;
    }

    /**
     * Gets a payment method's type image URL.
     *
     * @since 1.0.0
     *
     * @param string $type payment method type
     * @return string|null
     */
    public function get_payment_method_image_url($type)
    {
        $image_type = strtolower($type);

        if (is_readable($this->get_plugin()->get_plugin_path() . '/assets/images/card-' . $image_type . '.svg')) {
            return WC_HTTPS::force_https_url($this->get_plugin()->get_plugin_url() . '/assets/images/card-' . $image_type . '.svg');
        }

        return parent::get_payment_method_image_url($type);
    }
}
