<?php
/**
 * Poynt — a GoDaddy Brand for WooCommerce
 *
 * @author GoDaddy
 * @copyright Copyright (c) 2021 GoDaddy Operating Company, LLC. All Rights Reserved.
 * @license GPL-2.0
 */

namespace GoDaddy\WooCommerce\Poynt;

use GoDaddy\WooCommerce\Poynt\Frontend\MyAccount\PaymentMethods;
use GoDaddy\WooCommerce\Poynt\Gateways\CreditCardGateway;
use GoDaddy\WooCommerce\Poynt\Helpers\WPNUXHelper;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3 as Framework;
use SkyVerge\WooCommerce\PluginFramework\v5_10_3\SV_WC_Payment_Gateway;

defined('ABSPATH') or exit;

/**
 * Plugin main class.
 *
 * @since 1.0.0
 *
 * @method Lifecycle get_lifecycle_handler
 */
class Plugin extends Framework\SV_WC_Payment_Gateway_Plugin
{
    /** the plugin's version number */
    const VERSION = '1.1.2';

    /** the plugin's gateway id */
    const PLUGIN_ID = 'poynt';

    /** the plugin's production environment identifier */
    const ENVIRONMENT_PRODUCTION = 'production';

    /** the plugin's staging production identifier */
    const ENVIRONMENT_STAGING = 'staging';

    /** @var string the credit card gateway identifier */
    const CREDIT_CARD_GATEWAY_ID = 'poynt_credit_card';

    /** @var Plugin single instance of this plugin */
    protected static $instance;

    /**
     * Plugin constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        parent::__construct(
            self::PLUGIN_ID,
            self::VERSION,
            [
                'text_domain'  => 'godaddy-payments',
                'gateways'     => [
                    self::CREDIT_CARD_GATEWAY_ID => CreditCardGateway::class,
                ],
                // TODO: Replace currencies when limitations are clear (pitch still has a TODO instruction for this one). {AC 2021-02-01}
                'currencies'   => [],
                'dependencies' => [
                    'php_extensions' => [
                        'json',
                    ],
                ],
                'supports' => [
                    self::FEATURE_CAPTURE_CHARGE,
                    self::FEATURE_MY_PAYMENT_METHODS,
                ],
                'require_ssl' => true,
            ]
        );
    }

    /**
     * Gets the full path and filename of the plugin file.
     *
     * @since 1.0.0
     *
     * @return string
     */
    protected function get_file() : string
    {
        return __DIR__;
    }

    /**
     * Gets the plugin full name.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_plugin_name() : string
    {
        return __('Poynt &mdash; a GoDaddy Brand for WooCommerce', 'godaddy-payments');
    }

    /**
     * Gets the main plugin instance.
     *
     * Ensures only one instance is/can be loaded.
     *
     * @since 1.0.0
     *
     * @return Plugin
     */
    public static function instance() : self
    {
        if (null === self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Gets the plugin documentation URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_documentation_url() : string
    {
        // TODO: Replace with the documentation URL once it's available. {AC 2021-02-01}
        return '';
    }

    /**
     * Gets the reviews URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_reviews_url()
    {
        return 'https://wordpress.org/support/plugin/godaddy-payments/reviews/';
    }

    /**
     * Gets the plugin sales page URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_sales_page_url()
    {
        return 'https://www.poynt.com/';
    }

    /**
     * Gets the plugin support URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    public function get_support_url()
    {
        return 'https://wordpress.org/support/plugin/godaddy-payments/';
    }

    /**
     * Gets the my payment methods handler instance.
     *
     * @since 1.0.0
     *
     * @return PaymentMethods
     */
    protected function get_my_payment_methods_instance() : PaymentMethods
    {
        return new PaymentMethods($this);
    }

    /**
     * Loads and initializes the plugin lifecycle handler.
     *
     * @since 1.0.0
     */
    protected function init_lifecycle_handler()
    {
        $this->lifecycle_handler = new Lifecycle($this);
    }

    /**
     * Adds any admin notices upon initialization.
     *
     * @internal
     *
     * @since 1.0.0
     */
    public function add_admin_notices()
    {
        parent::add_admin_notices();

        if ($this->get_lifecycle_handler()->hasMigratedFromLegacyPlugin()) {

            $this->get_admin_notice_handler()->add_admin_notice(
                sprintf(
                    /* translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                    __('Migration successful! WooCommerce Poynt Collect Payment Gateway was deactivated, and Poynt &mdash; a GoDaddy Brand for WooCommerce has been %1$sconfigured with your previous settings%2$s.', 'godaddy-payments'),
                    '<a href="'.esc_url($this->get_settings_url()).'">', '</a>'
                ),
                Lifecycle::MIGRATED_FROM_LEGACY_PLUGIN_FLAG,
                [
                    'dismissible'             => true,
                    'always_show_on_settings' => false,
                    'notice_class'            => 'updated',
                ]
            );
        }
    }

    /**
     * Adds the delayed admin notices.
     *
     * @internal
     *
     * @since 1.0.0
     */
    public function add_delayed_admin_notices()
    {
        parent::add_delayed_admin_notices();

        $this->add_new_user_notices();
    }

    /**
     * Adds notices to prompt new user configuration.
     *
     * @since 1.0.0
     */
    private function add_new_user_notices()
    {
        /** @var CreditCardGateway $settings_gateway */
        $settings_gateway = $this->get_gateway(self::CREDIT_CARD_GATEWAY_ID);

        // warn and bail if the shop's currency isn't supported
        if ($this->maybeAddCurrencyAdminNotice()) {
            return;
        }

        // warn and bail if the shop's base isn't US
        if ($this->maybeAddCountryAdminNotice()) {
            return;
        }

        // only show the following notices on the gateway settings page
        if (! $settings_gateway || ! $this->is_payment_gateway_configuration_page($settings_gateway->get_id())) {
            return;
        }

        // show the signup prompt if the production credentials are empty
        $this->maybeAddSignupAdminNotice($settings_gateway);
    }

    /**
     * Gets the signup base URL.
     *
     * @since 1.1.2
     *
     * @return string
     */
    private function getSignupBaseUrl() : string
    {
        if (WPNUXHelper::isBHSite()) {
            return 'https://signup.poynt.com/r/bh';
        }

        if (WPNUXHelper::isCPanelSite()) {
            return 'https://signup.poynt.com/r/cpanel';
        }

        return 'https://signup.poynt.com/r/wordpress.org';
    }

    /**
     * Gets the signup URL.
     *
     * @since 1.0.0
     *
     * @return string
     */
    private function getSignupUrl() : string
    {
        $url = $this->getSignupBaseUrl();

        $user = wp_get_current_user();

        $args = array_filter(wc_clean([
            'name'                => $user ? trim($user->first_name.' '.$user->last_name) : '',
            'email'               => $user && is_email($user->user_email) ? $user->user_email : '',
            'companyName'         => get_bloginfo('name'),
            'companyAddressLine1' => WC()->countries->get_base_address(),
            'companyAddressLine2' => WC()->countries->get_base_address_2(),
            'companyAddressZip'   => WC()->countries->get_base_postcode(),
            'companyAddressCity'  => WC()->countries->get_base_city(),
            'companyAddressState' => WC()->countries->get_base_state(),
            'companyWebsite'      => esc_url(home_url()),
        ]));

        return add_query_arg($args, $url);
    }

    /**
     * Determines whether the current store country is supported or not.
     *
     * @since 1.1.0
     *
     * @return bool
     */
    public function isCountrySupported() : bool
    {
        return WC()->countries && 'US' === WC()->countries->get_base_country();
    }

    /**
     * Determines whether the current store currency is supported or not.
     *
     * @since 1.1.0
     *
     * @return bool
     */
    public function isCurrencySupported() : bool
    {
        return 'USD' === get_woocommerce_currency();
    }

    /**
     * May add an admin notice if the store's currency is not supported.
     *
     * @since 1.1.0
     *
     * @return bool true if the notice was added
     */
    private function maybeAddCurrencyAdminNotice() : bool
    {
        if ($this->isCurrencySupported()) {
            return false;
        }

        $this->get_admin_notice_handler()->add_admin_notice(
            sprintf(
                /** translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                __('Poynt &mdash; a GoDaddy Brand for WooCommerce requires U.S. dollar transactions. Please %1$schange your Currency%2$s in order to use Poynt.', 'godaddy-payments'),
                '<a href="'.esc_url($this->get_general_configuration_url()).'">', '</a>'
            ),
            'invalid-currency-prompt',
            [
                'notice_class' => 'notice-error',
            ]
        );

        return true;
    }

    /**
     * May add an admin notice if the store's country is not supported.
     *
     * @since 1.1.0
     *
     * @return bool true if the notice was added
     */
    private function maybeAddCountryAdminNotice() : bool
    {
        if ($this->isCountrySupported()) {
            return false;
        }

        $this->get_admin_notice_handler()->add_admin_notice(
            sprintf(
                /** translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                __('Poynt &mdash; a GoDaddy Brand for WooCommerce is available for United States-based businesses. Please %1$supdate your Store Address%2$s if you are located in the U.S.', 'godaddy-payments'),
                '<a href="'.esc_url($this->get_general_configuration_url()).'">', '</a>'
            ),
            'non-us-prompt',
            [
                'notice_class' => 'notice-error',
            ]
        );

        return true;
    }

    /**
     * May add an admin notice if the store's country is not supported.
     *
     * @since 1.1.0
     *
     * @param SV_WC_Payment_Gateway $settings_gateway the gateway settings
     */
    private function maybeAddSignupAdminNotice(SV_WC_Payment_Gateway $settings_gateway)
    {
        if (! $settings_gateway->getAppId(self::ENVIRONMENT_PRODUCTION) || ! $settings_gateway->getPrivateKey(self::ENVIRONMENT_PRODUCTION)) {

            $this->get_admin_notice_handler()->add_admin_notice(
                sprintf(
                    /** translators: Placeholders: %1$s - opening <a> HTML link tag, %2$s - closing </a> HTML link tag */
                    __('Start taking secure payments in minutes and get paid faster with Poynt &mdash; a GoDaddy Brand for WooCommerce, a payment gateway plugin that enables your US based business to accept any major credit or debit card directly on your WooCommerce site. %1$sSign Up Now%2$s', 'godaddy-payments'),
                    '<a href="'.esc_url($this->getSignupUrl()).'" target="_blank">', '</a>'
                ),
                'signup-prompt',
                [
                    'dismissible'             => true,
                    'always_show_on_settings' => false,
                ]
            );
        }
    }
}
